@@ -159,9 +159,6 @@ pub enum TestEventKind<'a> {
 
         /// The number of tests currently running, including this one.
         running: usize,
-
-        /// The cancel status of the run. This is None if the run is still ongoing.
-        cancel_state: Option<CancelReason>,
     },
 
     /// A test was slower than a configured soft timeout.
@@ -242,9 +239,6 @@ pub enum TestEventKind<'a> {
 
         /// The number of tests that are currently running, excluding this one.
         running: usize,
-
-        /// The cancel status of the run. This is None if the run is still ongoing.
-        cancel_state: Option<CancelReason>,
     },
 
     /// A test was skipped.
@@ -296,9 +290,6 @@ pub enum TestEventKind<'a> {
 
         /// The number of tests running.
         running: usize,
-
-        /// The cancel status of the run. This is None if the run is still ongoing.
-        cancel_reason: Option<CancelReason>,
     },
 
     /// A cancellation notice was received.
@@ -307,13 +298,12 @@ pub enum TestEventKind<'a> {
         setup_scripts_running: usize,
 
         /// Current statistics for number of tests so far.
+        ///
+        /// `current_stats.cancel_reason` is set to `Some`.
         current_stats: RunStats,
 
         /// The number of tests still running.
         running: usize,
-
-        /// The reason this run was cancelled.
-        reason: CancelReason,
     },
 
     /// A forcible kill was requested due to receiving a signal.
@@ -322,13 +312,12 @@ pub enum TestEventKind<'a> {
         setup_scripts_running: usize,
 
         /// Current statistics for number of tests so far.
+        ///
+        /// `current_stats.cancel_reason` is set to `Some`.
         current_stats: RunStats,
 
         /// The number of tests still running.
         running: usize,
-
-        /// The reason this run was killed.
-        reason: CancelReason,
     },
 
     /// A SIGTSTP event was received and the run was paused.
@@ -372,8 +361,8 @@ pub enum TestEventKind<'a> {
         /// The amount of time it took for the tests to run.
         elapsed: Duration,
 
-        /// Statistics for the run (last sub-run if this is a stress test).
-        run_stats: RunStats,
+        /// Statistics for the run, or overall statistics for stress tests.
+        run_stats: RunFinishedStats,
     },
 }
 
@@ -455,6 +444,27 @@ pub struct StressIndex {
     pub total: Option<NonZero<u32>>,
 }
 
+/// Statistics for a completed test run or stress run.
+#[derive(Clone, Debug)]
+pub enum RunFinishedStats {
+    /// A single test run was completed.
+    Single(RunStats),
+
+    /// A stress run was completed.
+    Stress(StressRunStats),
+}
+
+impl RunFinishedStats {
+    /// For a single run, returns a summary of statistics as an enum. For a
+    /// stress run, returns a summary for the last sub-run.
+    pub fn final_stats(&self) -> FinalRunStats {
+        match self {
+            Self::Single(stats) => stats.summarize_final(),
+            Self::Stress(stats) => stats.last_final_stats,
+        }
+    }
+}
+
 /// Statistics for a test run.
 #[derive(Copy, Clone, Default, Debug, Eq, PartialEq)]
 pub struct RunStats {
@@ -516,6 +526,9 @@ pub struct RunStats {
 
     /// The number of tests that were skipped.
     pub skipped: usize,
+
+    /// If the run is cancelled, the reason the cancellation is happening.
+    pub cancel_reason: Option<CancelReason>,
 }
 
 impl RunStats {
@@ -536,22 +549,48 @@ impl RunStats {
 
     /// Summarizes the stats as an enum at the end of a test run.
     pub fn summarize_final(&self) -> FinalRunStats {
-        // Check for failures first. The order of setup scripts vs tests should not be important,
-        // though we don't assert that here.
+        // Check for failures first. The order of setup scripts vs tests should
+        // not be important, though we don't assert that here.
         if self.failed_setup_script_count() > 0 {
-            FinalRunStats::Failed(RunStatsFailureKind::SetupScript)
+            // Is this related to a cancellation other than one directly caused
+            // by the failure?
+            if self.cancel_reason > Some(CancelReason::TestFailure) {
+                FinalRunStats::Cancelled {
+                    reason: self.cancel_reason,
+                    kind: RunStatsFailureKind::SetupScript,
+                }
+            } else {
+                FinalRunStats::Failed(RunStatsFailureKind::SetupScript)
+            }
         } else if self.setup_scripts_initial_count > self.setup_scripts_finished_count {
-            FinalRunStats::Cancelled(RunStatsFailureKind::SetupScript)
+            FinalRunStats::Cancelled {
+                reason: self.cancel_reason,
+                kind: RunStatsFailureKind::SetupScript,
+            }
         } else if self.failed_count() > 0 {
-            FinalRunStats::Failed(RunStatsFailureKind::Test {
+            let kind = RunStatsFailureKind::Test {
                 initial_run_count: self.initial_run_count,
                 not_run: self.initial_run_count.saturating_sub(self.finished_count),
-            })
+            };
+
+            // Is this related to a cancellation other than one directly caused
+            // by the failure?
+            if self.cancel_reason > Some(CancelReason::TestFailure) {
+                FinalRunStats::Cancelled {
+                    reason: self.cancel_reason,
+                    kind,
+                }
+            } else {
+                FinalRunStats::Failed(kind)
+            }
         } else if self.initial_run_count > self.finished_count {
-            FinalRunStats::Cancelled(RunStatsFailureKind::Test {
-                initial_run_count: self.initial_run_count,
-                not_run: self.initial_run_count.saturating_sub(self.finished_count),
-            })
+            FinalRunStats::Cancelled {
+                reason: self.cancel_reason,
+                kind: RunStatsFailureKind::Test {
+                    initial_run_count: self.initial_run_count,
+                    not_run: self.initial_run_count.saturating_sub(self.finished_count),
+                },
+            }
         } else if self.finished_count == 0 {
             FinalRunStats::NoTestsRun
         } else {
@@ -648,12 +687,67 @@ pub enum FinalRunStats {
     NoTestsRun,
 
     /// The test run was cancelled.
-    Cancelled(RunStatsFailureKind),
+    Cancelled {
+        /// The reason for cancellation, if available.
+        ///
+        /// This should generally be available, but may be None if some tests
+        /// that were selected to run were not executed.
+        reason: Option<CancelReason>,
+
+        /// The kind of failure that occurred.
+        kind: RunStatsFailureKind,
+    },
 
     /// At least one test failed.
     Failed(RunStatsFailureKind),
 }
 
+/// Statistics for a stress run.
+#[derive(Clone, Debug)]
+pub struct StressRunStats {
+    /// The number of stress runs completed.
+    pub completed: StressIndex,
+
+    /// The number of stress runs that succeeded.
+    pub success_count: u32,
+
+    /// The number of stress runs that failed.
+    pub failed_count: u32,
+
+    /// The last stress run's `FinalRunStats`.
+    pub last_final_stats: FinalRunStats,
+}
+
+impl StressRunStats {
+    /// Summarizes the stats as an enum at the end of a test run.
+    pub fn summarize_final(&self) -> StressFinalRunStats {
+        if self.failed_count > 0 {
+            StressFinalRunStats::Failed
+        } else if matches!(self.last_final_stats, FinalRunStats::Cancelled { .. }) {
+            StressFinalRunStats::Cancelled
+        } else if matches!(self.last_final_stats, FinalRunStats::NoTestsRun) {
+            StressFinalRunStats::NoTestsRun
+        } else {
+            StressFinalRunStats::Success
+        }
+    }
+}
+
+/// A summary of final statistics for a stress run.
+pub enum StressFinalRunStats {
+    /// The stress run was successful.
+    Success,
+
+    /// No tests were run.
+    NoTestsRun,
+
+    /// The stress run was cancelled.
+    Cancelled,
+
+    /// At least one stress run failed.
+    Failed,
+}
+
 /// A type summarizing the step at which a test run failed.
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub enum RunStatsFailureKind {
@@ -1376,10 +1470,13 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Cancelled(RunStatsFailureKind::Test {
-                initial_run_count: 42,
-                not_run: 1
-            }),
+            FinalRunStats::Cancelled {
+                reason: None,
+                kind: RunStatsFailureKind::Test {
+                    initial_run_count: 42,
+                    not_run: 1
+                }
+            },
             "initial run count > final run count => cancelled"
         );
         assert_eq!(
@@ -1443,7 +1540,10 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Cancelled(RunStatsFailureKind::SetupScript),
+            FinalRunStats::Cancelled {
+                reason: None,
+                kind: RunStatsFailureKind::SetupScript,
+            },
             "setup script failed => failure"
         );
 
