@@ -16,8 +16,9 @@ use crate::{
     input::{InputEvent, InputHandler},
     list::{TestInstance, TestInstanceId, TestList},
     reporter::events::{
-        CancelReason, ExecuteStatus, ExecutionStatuses, InfoResponse, RunStats, StressIndex,
-        StressProgress, TestEvent, TestEventKind,
+        CancelReason, ExecuteStatus, ExecutionStatuses, FinalRunStats, InfoResponse,
+        RunFinishedStats, RunStats, StressIndex, StressProgress, StressRunStats, TestEvent,
+        TestEventKind,
     },
     runner::{ExecutorEvent, RunUnitQuery, SignalRequest, StressCondition, StressCount},
     signal::{JobControlEvent, ShutdownEvent, SignalEvent, SignalHandler, SignalInfoEvent},
@@ -51,7 +52,6 @@ pub(super) struct DispatcherContext<'a, F> {
     global_timeout: Duration,
     running_setup_script: Option<ContextSetupScript<'a>>,
     running_tests: BTreeMap<TestInstanceId<'a>, ContextTestInstance<'a>>,
-    cancel_state: Option<CancelReason>,
     signal_count: Option<SignalCount>,
     stress_cx: DispatcherStressContext,
     #[cfg(test)]
@@ -87,7 +87,6 @@ where
             global_timeout,
             running_setup_script: None,
             running_tests: BTreeMap::new(),
-            cancel_state: None,
             signal_count: None,
             stress_cx: DispatcherStressContext::new(stress_condition),
             #[cfg(test)]
@@ -364,7 +363,9 @@ where
     }
 
     pub(super) fn stress_sub_run_finished(&mut self) {
-        let sub_elapsed = self.stress_cx.mark_completed();
+        let sub_elapsed = self
+            .stress_cx
+            .mark_completed(self.run_stats.summarize_final());
         let progress = self
             .stress_progress()
             .expect("stress_sub_run_finished called in non-stress test context");
@@ -386,7 +387,7 @@ where
 
     /// Returns the reason for cancellation, or `None` if the run is not cancelled.
     pub(super) fn cancel_reason(&self) -> Option<CancelReason> {
-        self.cancel_state
+        self.run_stats.cancel_reason
     }
 
     #[inline]
@@ -421,7 +422,7 @@ where
                 total,
                 req_rx_tx,
             }) => {
-                if self.cancel_state.is_some() {
+                if self.run_stats.cancel_reason.is_some() {
                     // The run has been cancelled: don't start any new units.
                     return HandleEventResponse::None;
                 }
@@ -500,7 +501,7 @@ where
                 test_instance,
                 req_rx_tx,
             }) => {
-                if self.cancel_state.is_some() {
+                if self.run_stats.cancel_reason.is_some() {
                     // The run has been cancelled: don't start any new units.
                     return HandleEventResponse::None;
                 }
@@ -520,7 +521,6 @@ where
                     test_instance,
                     current_stats: self.run_stats,
                     running: self.running_tests.len(),
-                    cancel_state: self.cancel_state,
                 })
             }
             InternalEvent::Executor(ExecutorEvent::Slow {
@@ -559,7 +559,7 @@ where
                 retry_data,
                 tx,
             }) => {
-                if self.cancel_state.is_some() {
+                if self.run_stats.cancel_reason.is_some() {
                     // The run has been cancelled: don't send a message over the tx and don't start
                     // any new units.
                     return HandleEventResponse::None;
@@ -605,7 +605,6 @@ where
                     run_statuses,
                     current_stats: self.run_stats,
                     running: self.running(),
-                    cancel_state: self.cancel_state,
                 });
 
                 if fail_cancel {
@@ -639,7 +638,6 @@ where
                 self.callback_none_response(TestEventKind::InputEnter {
                     current_stats: self.run_stats,
                     running: self.running(),
-                    cancel_reason: self.cancel_state,
                 })
             }
             InternalEvent::ReportCancel => {
@@ -859,21 +857,19 @@ where
         if event == CancelEvent::Signal(ShutdownRequest::Twice) {
             // Forcibly kill child processes in the case of a second shutdown
             // signal.
+            self.run_stats.cancel_reason = Some(CancelReason::SecondSignal);
             self.basic_callback(TestEventKind::RunBeginKill {
                 setup_scripts_running: self.setup_scripts_running(),
                 current_stats: self.run_stats,
                 running: self.running(),
-                // This is always a second signal.
-                reason: CancelReason::SecondSignal,
             });
             HandleEventResponse::Cancel(event)
-        } else if self.cancel_state < Some(reason) {
-            self.cancel_state = Some(reason);
+        } else if self.run_stats.cancel_reason < Some(reason) {
+            self.run_stats.cancel_reason = Some(reason);
             self.basic_callback(TestEventKind::RunBeginCancel {
                 setup_scripts_running: self.setup_scripts_running(),
                 current_stats: self.run_stats,
                 running: self.running(),
-                reason,
             });
             HandleEventResponse::Cancel(event)
         } else {
@@ -887,7 +883,13 @@ where
             start_time: stopwatch_end.start_time.fixed_offset(),
             run_id: self.run_id,
             elapsed: stopwatch_end.active,
-            run_stats: self.run_stats,
+            run_stats: self
+                .stress_cx
+                .run_stats(self.run_stats.summarize_final())
+                .map_or_else(
+                    || RunFinishedStats::Single(self.run_stats),
+                    RunFinishedStats::Stress,
+                ),
         })
     }
 
@@ -903,6 +905,8 @@ enum DispatcherStressContext {
         condition: StressCondition,
         sub_stopwatch: StopwatchStart,
         completed: u32,
+        failed: u32,
+        cancelled: bool,
     },
 }
 
@@ -913,6 +917,8 @@ impl DispatcherStressContext {
                 condition,
                 sub_stopwatch: crate::time::stopwatch(),
                 completed: 0,
+                failed: 0,
+                cancelled: false,
             }
         } else {
             Self::None
@@ -933,6 +939,8 @@ impl DispatcherStressContext {
                 condition,
                 sub_stopwatch: _,
                 completed,
+                failed: _,
+                cancelled: _,
             } => match condition {
                 StressCondition::Count(total) => Some(StressProgress::Count {
                     total: *total,
@@ -970,7 +978,7 @@ impl DispatcherStressContext {
         }
     }
 
-    fn mark_completed(&mut self) -> Duration {
+    fn mark_completed(&mut self, summary: FinalRunStats) -> Duration {
         match self {
             Self::None => {
                 panic!("mark_completed called in a non-stress test context");
@@ -979,8 +987,26 @@ impl DispatcherStressContext {
                 condition: _,
                 sub_stopwatch,
                 completed,
+                failed,
+                cancelled,
             } => {
                 *completed += 1;
+                match summary {
+                    FinalRunStats::Success => {}
+                    FinalRunStats::NoTestsRun => {
+                        // TODO: We should figure out whether to terminate the
+                        // test run based on this.
+                    }
+                    FinalRunStats::Failed(_) => {
+                        *failed += 1;
+                    }
+                    FinalRunStats::Cancelled { .. } => {
+                        // In this case, we don't add to the failed count. The
+                        // displayer will take care of displaying the
+                        // cancellation message properly.
+                        *cancelled = true;
+                    }
+                }
                 let duration = sub_stopwatch.snapshot().active;
                 *sub_stopwatch = crate::time::stopwatch();
                 duration
@@ -988,6 +1014,32 @@ impl DispatcherStressContext {
         }
     }
 
+    fn run_stats(&self, last_final_stats: FinalRunStats) -> Option<StressRunStats> {
+        match self {
+            Self::None => None,
+            Self::Stress {
+                condition: _,
+                sub_stopwatch: _,
+                completed,
+                failed,
+                cancelled,
+            } => {
+                let mut success_count = completed.saturating_sub(*failed);
+                // If the run is cancelled, there's one less success than we
+                // thought above.
+                if *cancelled {
+                    success_count = success_count.saturating_sub(1);
+                }
+                Some(StressRunStats {
+                    completed: self.stress_index().expect("we're in the Self::Stress case"),
+                    success_count,
+                    failed_count: *failed,
+                    last_final_stats,
+                })
+            }
+        }
+    }
+
     #[cfg(unix)]
     fn pause_stopwatch(&mut self) {
         match self {
@@ -1150,16 +1202,19 @@ mod tests {
             let event = events.pop().unwrap();
             let TestEventKind::RunBeginCancel {
                 setup_scripts_running,
-                current_stats: _,
+                current_stats,
                 running,
-                reason,
             } = event.kind
             else {
                 panic!("expected RunBeginCancel event, found {:?}", event.kind);
             };
             assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
             assert_eq!(running, 0, "expected 0 tests running");
-            assert_eq!(reason, CancelReason::ReportError, "expected report error");
+            assert_eq!(
+                current_stats.cancel_reason,
+                Some(CancelReason::ReportError),
+                "expected report error"
+            );
         }
 
         // Send another report error, ensuring it's ignored.
@@ -1195,16 +1250,19 @@ mod tests {
                     let event = events.pop().unwrap();
                     let TestEventKind::RunBeginCancel {
                         setup_scripts_running,
-                        current_stats: _,
+                        current_stats,
                         running,
-                        reason,
                     } = event.kind
                     else {
                         panic!("expected RunBeginCancel event, found {:?}", event.kind);
                     };
                     assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
                     assert_eq!(running, 0, "expected 0 tests running");
-                    assert_eq!(reason, event_to_cancel_reason(*sig1), "expected signal");
+                    assert_eq!(
+                        current_stats.cancel_reason,
+                        Some(event_to_cancel_reason(*sig1)),
+                        "expected signal"
+                    );
                 }
 
                 // Another report error, ensuring it's ignored.
@@ -1224,16 +1282,19 @@ mod tests {
                     let event = events.pop().unwrap();
                     let TestEventKind::RunBeginKill {
                         setup_scripts_running,
-                        current_stats: _,
+                        current_stats,
                         running,
-                        reason,
                     } = event.kind
                     else {
                         panic!("expected RunBeginKill event, found {:?}", event.kind);
                     };
                     assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
                     assert_eq!(running, 0, "expected 0 tests running");
-                    assert_eq!(reason, CancelReason::SecondSignal, "expected second signal");
+                    assert_eq!(
+                        current_stats.cancel_reason,
+                        Some(CancelReason::SecondSignal),
+                        "expected second signal"
+                    );
                 }
 
                 // Another report error, ensuring it's ignored.
